import 'package:device_preview/device_preview.dart';
import 'package:flutter/material.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:global_configuration/global_configuration.dart';
import 'package:provider/provider.dart';
import 'package:training_lights/global/notify_listeners/language_manager/language_manager.dart';
import 'package:training_lights/global/notify_listeners/language_manager/localizations_delegate.dart';
import 'package:training_lights/my_theme.dart';
import 'package:training_lights/route_generator.dart';
import 'package:training_lights/utils/languages/language.dart';
import 'package:training_lights/utils/logger.dart';

class App extends StatelessWidget {
  final log = getLogger('App');

  final String appName = GlobalConfiguration().getValue('appName');
  final bool devicePreview = GlobalConfiguration().getValue('devicePreview');

  final List<Locale> supportedLocales = Language.allSupportedLanguages().map((language) {
    return Locale(language.languageCode);
  }).toList();

  static const List<LocalizationsDelegate> localizationsDelegates = [
    AppLocalizationsDelegate(),
    GlobalMaterialLocalizations.delegate,
    GlobalWidgetsLocalizations.delegate,
    GlobalCupertinoLocalizations.delegate,
  ];

  Locale? localeListResolutionCallback(List<Locale>? locales, Iterable<Locale> supportedLocales) {
    for (final supportedLocale in supportedLocales) {
      for (final locale in locales!) {
        if (supportedLocale.languageCode == locale.languageCode) {
          return supportedLocale;
        }
      }
    }
    // if locale language isn't supported use english
    return const Locale('en');
  }

  @override
  Widget build(BuildContext context) {
    return Consumer<LanguageManager>(
      builder: (_, languageNotifier, __) {
        if (devicePreview) {
          log.i('Building the core App widget with DevicePreview');
          return DevicePreview(
            builder: (context) =>
                MaterialApp(
                  title: appName,
                  theme: myTheme(),
                  locale: DevicePreview.locale(context),
                  builder: DevicePreview.appBuilder,
                  initialRoute: '/',
                  onGenerateRoute: RouteGenerator.generate,
                  supportedLocales: supportedLocales,
                  localizationsDelegates: localizationsDelegates,
                  localeListResolutionCallback: localeListResolutionCallback,
                ),
          );
        } else {
          log.i('Building the core App widget');
          return Consumer<LanguageManager>(
            builder: (_, languageNotifier, __) {
              return MaterialApp(
                title: appName,
                builder: (BuildContext context, Widget? widget) {
                  ErrorWidget.builder = (FlutterErrorDetails errorDetails) {
                    return Text(
                      Language.of(context).appSomethingWrong,
                      style: myTheme().textTheme.headline1,
                    );
                  };
                  return Theme(
                    data: myTheme(),
                    child: widget!,
                  );
                },
                initialRoute: '/',
                onGenerateRoute: RouteGenerator.generate,
                locale: languageNotifier.locale,
                supportedLocales: supportedLocales,
                localizationsDelegates: localizationsDelegates,
                localeListResolutionCallback: localeListResolutionCallback,
              );
            },
          );
        }
      },
    );
  }
}
