import 'dart:math';

import 'package:flutter/material.dart';
import 'package:training_lights/common/buttons/empty_appbar_action.dart';
import 'package:training_lights/common/buttons/leading_back_button.dart';
import 'package:training_lights/common/video/video_aspect_ratio.dart';
import 'package:training_lights/my_theme.dart';
import 'package:training_lights/utils/show_dialog.dart';

class MyScaffold extends StatelessWidget {
  final Widget body;
  final bool isBackButtonDisplayed;
  final bool isBottomGradientDisplayed;
  final bool isPairingRouteGradientDisplay;
  final bool sliverAppBar;
  final bool alignTitleLeft;
  final bool zeroBodyPadding;
  final Widget? willPopDialog;
  final Widget? title;
  final Widget? actionButton;
  final Widget? bottomNavigationBar;
  final Widget? backgroundSliverAppBar;
  final Color? scaffoldBackgroundColor;
  final Color? appBarWidgetsBackgroundColor;

  const MyScaffold({
    required this.body,
    required this.isBackButtonDisplayed,
    required this.isBottomGradientDisplayed,
    this.isPairingRouteGradientDisplay = false,
    this.sliverAppBar = false,
    this.alignTitleLeft = false,
    this.zeroBodyPadding = false,
    this.willPopDialog,
    this.title,
    this.actionButton,
    this.bottomNavigationBar,
    this.backgroundSliverAppBar,
    this.scaffoldBackgroundColor,
    this.appBarWidgetsBackgroundColor,
  });

  @override
  Widget build(BuildContext context) {
    final theme = myTheme();
    final size = MediaQuery.of(context).size;
    // if this padding would be changed, it will have to be changed also in HomeRoute
    const padding = EdgeInsets.symmetric(horizontal: 20);
    const bottomSafeArea = false; // because of home indicator on iPhones

    final centeredAppBar = Column( // Column has to be here because of center AppBar vertically
      mainAxisAlignment: backgroundSliverAppBar != null ?
        MainAxisAlignment.start : MainAxisAlignment.center,
      children: [
        Padding(
          padding: padding,
          child: AppBar(
            backgroundColor: Colors.transparent,
            toolbarHeight: 40,
            elevation: 0,
            leading: isBackButtonDisplayed ? LeadingBackButton(
              willPopDialog: willPopDialog,
              backgroundColor: appBarWidgetsBackgroundColor,
            ) : Container(height: 0,),
            leadingWidth: alignTitleLeft ? 0 : null,
            titleSpacing: alignTitleLeft ? 0 : null,
            centerTitle: !alignTitleLeft,
            title: title ?? Container(height: 0),
            actions: [
              actionButton ?? EmptyAppbarAction(),
            ],
          ),
        )
      ],
    );

    Widget scaffoldBody = Padding(
      padding: zeroBodyPadding ? EdgeInsets.zero : padding,
      child: Center(
          child: body,
      ),
    );

    if (sliverAppBar) {
      scaffoldBody = NestedScrollView(
          body: scaffoldBody,
          headerSliverBuilder: (BuildContext context, _) => [
            SliverOverlapAbsorber(
              sliver: SliverSafeArea(
                bottom: bottomSafeArea,
                top: false,
                sliver: SliverAppBar(
                  pinned: true,
                  floating: true,
                  backgroundColor: Colors.transparent,
                  expandedHeight: _getHeightOfAppBar(size),
                  automaticallyImplyLeading: false,
                  flexibleSpace: backgroundSliverAppBar != null ? Stack(
                    children: [
                      backgroundSliverAppBar!,
                      Padding(
                        padding: const EdgeInsets.only(top: 10),
                        child: centeredAppBar,
                      )
                    ],
                  ) : centeredAppBar,
                ),
              ),
              handle: NestedScrollView.sliverOverlapAbsorberHandleFor(context),
            )
          ],
      );
    }

    return WillPopScope(
        onWillPop: () async {
          if (willPopDialog == null) {
            return true;
          } else {
            ShowDialog.show(
                context: context,
                child: willPopDialog!,
            );
            return false;
          }
        },
        child: Container(
            height: size.height,
            width: size.width,
            decoration: isPairingRouteGradientDisplay ? BoxDecoration(
                gradient: RadialGradient(
                    colors: [
                      theme.colorScheme.secondary.withOpacity(0.3),
                      Colors.transparent
                    ],
                    focal: const Alignment(0,-0.275), // depends on position of status layout in pairing route widget
                    radius: 1.5,
                ),
            ) : null,
            child: SafeArea(
                bottom: bottomSafeArea,
                child: Scaffold(
                  bottomNavigationBar: bottomNavigationBar,
                  backgroundColor: scaffoldBackgroundColor,
                  appBar: !sliverAppBar ? PreferredSize(
                      preferredSize: Size.fromHeight(size.height * 0.15),
                      child: centeredAppBar,
                  ) : null,
                  body: isBottomGradientDisplayed ? BottomGradient(
                      body: scaffoldBody,
                  ) : scaffoldBody,
                ),
            ),
        ),
    );
  }

  double _getHeightOfAppBar(Size size) {
    if (backgroundSliverAppBar == null) {
      return size.height * 0.15;
    } else {
      return min(size.width / VideoAspectRatio.getAspectRatioForVideo(), size.height*0.6);
    }
  }
}

class BottomGradient extends StatelessWidget {
  final Widget body;

  const BottomGradient({required this.body});

  @override
  Widget build(BuildContext context) {
    final size = MediaQuery.of(context).size;

    return Container(
        height: size.height,
        width: size.width,
        decoration: BoxDecoration(
            gradient: RadialGradient(
                colors: [
                  myTheme().colorScheme.secondary.withOpacity(0.4),
                  Colors.transparent
                ],
                focal: Alignment.bottomCenter,
                radius: 1.7,
                center: const Alignment(0, 1.5),
            ),
        ),
        child: body,
    );
  }
}
