import 'package:flutter/material.dart';
import 'package:flutter_cache_manager/flutter_cache_manager.dart';
import 'package:global_configuration/global_configuration.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:sqflite/sqflite.dart';
import 'package:tangle_connector/tangle_connector.dart';
import 'package:tangle_utils/tangle_utils.dart';
import 'package:training_lights/app.dart';
import 'package:training_lights/global/notify_listeners/battery_manager.dart';
import 'package:training_lights/global/notify_listeners/language_manager/language_manager.dart';
import 'package:training_lights/global/notify_listeners/notifications_manager.dart';
import 'package:training_lights/global/notify_listeners/scanner_state_manager.dart';
import 'package:training_lights/global/notify_listeners/tangle_manager/tangle_delivery.dart';
import 'package:training_lights/global/notify_listeners/tangle_manager/tangle_manager.dart';
import 'package:training_lights/global/notify_listeners/training_manager.dart';
import 'package:training_lights/global/services.dart';
import 'package:training_lights/global/services/database_service/adopted_devices_service.dart';
import 'package:training_lights/global/services/database_service/database_initialization.dart';
import 'package:training_lights/global/services/database_service/training_service.dart';
import 'package:training_lights/global/services/shared_preferences_service.dart';
import 'package:training_lights/global/services/video_controller_service.dart';
import 'package:training_lights/global/test.dart';
import 'package:training_lights/utils/logger.dart';
import 'package:training_lights/utils/system_settings/system_settings.dart';
import 'package:training_lights/utils/tangle_generator/tangle_generator.dart';

Future<void> main() async {
  WidgetsFlutterBinding.ensureInitialized();
  await GlobalConfiguration().loadFromAsset('config.json');
  SystemSettings.setEnabledOrientations();
  SystemSettings.setPhoneBarsColors();

  runApp(AppStarter(
    database: await DatabaseInitialization.initDatabase(),
    sharedPrefs: await SharedPreferences.getInstance(),
  ),);
}

class AppStarter extends StatelessWidget {
  final log = getLogger('AppStarter');
  final String firmwareVersion = GlobalConfiguration().getValue('firmwareVersion');
  final String firmwarePath = GlobalConfiguration().getValue('firmwarePath');
  final int scanDuration = GlobalConfiguration().getValue('scanTimeout');
  final int writeTimeout = GlobalConfiguration().getValue('writeTimeout');
  final String devicePrefix = GlobalConfiguration().getValue('devicePrefix');
  final String serviceUUID = GlobalConfiguration().getValue('serviceUUID');
  final String networkCharacteristicUUID = GlobalConfiguration().getValue('networkCharacteristicUUID');
  final String clockCharacteristicUUID = GlobalConfiguration().getValue('clockCharacteristicUUID');
  final String deviceCharacteristicUUID = GlobalConfiguration().getValue('deviceCharacteristicUUID');

  final String networkFilesUrl = GlobalConfiguration().getValue('networkFilesUrl');
  final int connectionTimeoutPerAttempt = GlobalConfiguration().getValue('connectionTimeoutPerAttempt');
  final int connectionRetryAttempts = GlobalConfiguration().getValue('connectionRetryAttempts');

  final Database database;
  final SharedPreferences sharedPrefs;

  AppStarter({
    required this.database,
    required this.sharedPrefs,
  });

  @override
  Widget build(BuildContext context) {
    final adoptedDevicesService = AdoptedDevicesServiceImpl(database: database);
    final scannerStateManager = ScannerStateManagerImpl();
    final batteryManager = BatteryManagerImpl();
    final bluetoothService = BluetoothConnectionService.instance(
      firmwareVersion: firmwareVersion,
      scanDuration: scanDuration,
      writeTimeout: writeTimeout,
      serviceUUID: serviceUUID,
      networkCharacteristicUUID: networkCharacteristicUUID,
      clockCharacteristicUUID: clockCharacteristicUUID,
      deviceCharacteristicUUID: deviceCharacteristicUUID,
      discoverServicesTimeout: (0.9 * connectionTimeoutPerAttempt).toInt(),
    );
    final gpsService = GpsService.instance();
    final videoControllerService = VideoServiceImpl(
      networkFilesUrl: networkFilesUrl,
      cacheManager: DefaultCacheManager(),
    );
    final sharedPrefsService = SharedPreferencesServiceImpl(storage: sharedPrefs);
    sharedPrefsService.incrementOpenAppNumber();

    final databaseService = TrainingServiceImpl(database: database);

    final tangleGenerator = TangleGeneratorImpl(builder: TangleBuilder());
    final tangleDelivery = TangleDeliveryImpl(
      batteryManager: batteryManager,
      codeParser: TangleCodeParser(),
      metadataParser: TangleMetadataParser(),
    );
    final tangleManager = TangleManagerImpl(
      bluetoothConnectionService: bluetoothService,
      adoptedDevicesService: adoptedDevicesService,
      scannerManager: scannerStateManager,
      generator: tangleGenerator,
      delivery: tangleDelivery,
      ownerSignature: sharedPrefsService.getOwnerSignature(), // TODO - hardcoded in service
      ownerKey: sharedPrefsService.getOwnerKey(), // TODO - hardcoded in service
      firmwarePath: firmwarePath,
      devicePrefix: devicePrefix,
      connectionTimeoutPerAttempt: connectionTimeoutPerAttempt,
      connectionRetryAttempts: connectionRetryAttempts,
    );
    final TrainingManager trainingManager = TrainingManagerImpl(db: databaseService);
    final LanguageManager languageManager = LanguageManager(sharedPrefs: sharedPrefsService);
    final NotificationsManager notificationsManager = NotificationsManager(sharedPrefs: sharedPrefsService);

    log.i('Building AppStarter widget');
    return GestureDetector(
      onTap: () {
        // Hide keyboard if tapped outside of form
        final FocusScopeNode currentFocus = FocusScope.of(context);
        if (!currentFocus.hasPrimaryFocus) {
          currentFocus.focusedChild!.unfocus();
        }
      },
      child: Test(
        isTested: false,
        child: MultiProvider(
          providers: [
            ChangeNotifierProvider<TangleManager>(create: (_) => tangleManager),
            ChangeNotifierProvider<ScannerStateManager>(create: (_) => scannerStateManager),
            ChangeNotifierProvider<LanguageManager>(create: (_) => languageManager),
            ChangeNotifierProvider<TrainingManager>(create: (_) => trainingManager),
            ChangeNotifierProvider<NotificationsManager>(create: (_) => notificationsManager),
            ChangeNotifierProvider<BatteryManager>(create: (_) => batteryManager),
            ChangeNotifierProvider<Services>(create: (_) => ServicesImpl(
              bluetooth: bluetoothService,
              gps: gpsService,
              video: videoControllerService,
              sharedPrefs: sharedPrefsService,
            ),),
          ],
          child: App(),
        ),
      ),
    );
  }
}
