import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:training_lights/common/buttons/primary_button.dart';
import 'package:training_lights/common/dialogs/will_pop_dialog.dart';
import 'package:training_lights/common/my_icons/my_icons.dart';
import 'package:training_lights/common/wrappers/my_scaffold.dart';
import 'package:training_lights/global/enums/primary_button_design.dart';
import 'package:training_lights/global/notify_listeners/tangle_manager/tangle_manager.dart';
import 'package:training_lights/models/training_model/training.dart';
import 'package:training_lights/my_theme.dart';
import 'package:training_lights/routes/connection_lost_route.dart';
import 'package:training_lights/routes/countdown_route/countdown_controller.dart';
import 'package:training_lights/routes/countdown_route/countdown_state.dart';
import 'package:training_lights/routes/countdown_route/widgets/circular_countdown_timer.dart';
import 'package:training_lights/routes/countdown_route/widgets/dialogs/restart_dialog.dart';
import 'package:training_lights/routes/countdown_route/widgets/text_title.dart';
import 'package:training_lights/utils/languages/language.dart';
import 'package:training_lights/utils/show_dialog.dart';

class CountdownRoute extends StatefulWidget {
  final TrainingModel trainingModel;

  const CountdownRoute({
    required this.trainingModel,
  });

  @override
  State<StatefulWidget> createState() => CountdownRouteState();
}

class CountdownRouteState extends State<CountdownRoute>{
  late CountdownController controller;
  late CountdownState countdownState;
  bool firstStartDone = false;

  @override
  void initState() {
    controller = CountdownController();
    countdownState = CountdownState.paused;
    super.initState();
  }
  
  void setStateOnCompleted() {
    setState(() {
      countdownState = CountdownState.completed;
    });
  }

  void setStateOnPaused() {
    setState(() {
      countdownState = CountdownState.paused;
    });
  }

  void setStateOnRunning() {
    setState(() {
      countdownState = CountdownState.running;
    });
  }

  bool wasCompleted() {
    return countdownState == CountdownState.completed;
  }

  void restart(TangleManager tangle) {
    tangle.resetTimeline();
    controller.restart();
    setStateOnRunning();
  }

  @override
  Widget build(BuildContext context) {
    final theme = myTheme();
    final language = Language.of(context);
    const IconData restartIcon = Icons.replay;
    final appBarWidgetsBackgroundColor = theme.primaryColor.withOpacity(0.2);

    final Widget willPopDialog = Consumer<TangleManager>(
      builder: (_, tangle, __) => WillPopDialog(
        contentText: language.countdownRoutePopDialogLeave,
        beforePop: tangle.endTraining,
      ),
    );

    return Consumer<TangleManager>(
        builder: (_, tangle, __) {
          if (tangle.isReady) {
            return MyScaffold(
              isBackButtonDisplayed: true,
              isBottomGradientDisplayed: false,
              actionButton: _restartIconButton(restartIcon),
              scaffoldBackgroundColor: theme.colorScheme.secondary,
              willPopDialog: !wasCompleted() ? willPopDialog : null,
              title: TextTitle(
                title: widget.trainingModel.name,
                backgroundColor: appBarWidgetsBackgroundColor,
              ),
              appBarWidgetsBackgroundColor: appBarWidgetsBackgroundColor,
              body: Column(
                children: [
                  Expanded(
                    flex: 3,
                    child: Center(
                      child: CircularCountDownTimer(
                        trainingModel: widget.trainingModel,
                        countDownController: controller,
                        onComplete: setStateOnCompleted,
                      ),
                    ),
                  ),
                  Flexible(
                      child: Center(
                          child: _setCountdownStateButton(restartIcon),
                      ),
                  )
                ],
              ),
            );
          } else {
            return ConnectionLostRoute();
          }
        },
    );
  }

  Widget _restartIconButton(IconData restartIcon) {
    return Container(
      margin: const EdgeInsets.only(left: 15),
      decoration: BoxDecoration(
        color: myTheme().primaryColor.withOpacity(0.2),
        shape: BoxShape.circle,
      ),
      child: Consumer<TangleManager>(
        builder: (_, tangle, __) {
          return IconButton(
              splashRadius: 20,
              onPressed: () {
                if (wasCompleted()) {
                  restart(tangle);
                } else {
                  ShowDialog.show(
                      context: context,
                      child: RestartDialog(
                          onYes: () {
                            restart(tangle);
                            Navigator.pop(context);
                          },
                      ),
                  );
                }
              },
              icon: Icon(restartIcon),
          );
        },
      ),
    );
  }

  Widget _setCountdownStateButton(IconData restartIcon) {
    return Consumer<TangleManager>(
      builder: (_, tangle, __) {
        final language = Language.of(context);
        Function() onPressed;
        IconData icon;
        String text;

        switch(countdownState) {
          case CountdownState.running:
            onPressed = () {
              tangle.pauseTimeline();
              controller.pause();
              setStateOnPaused();
            };
            icon = MyIcons.pause;
            text = language.countdownRouteStop;
            break;
          case CountdownState.paused:
            onPressed = () {
              if (!firstStartDone) {
                firstStartDone = true;
                tangle.startTraining();
              } else {
                tangle.resumeTimeline();
              }
              controller.resume();
              setStateOnRunning();
            };
            icon = MyIcons.play;
            text = firstStartDone ? language.countdownRouteContinue : language.countdownRoutePlay;
            break;
          case CountdownState.completed:
            onPressed = () => restart(tangle);
            icon = restartIcon;
            text = language.countdownRouteRunAgain;
            break;
        }

        return PrimaryButton(
            buttonDesign: PrimaryButtonDesign.darkColorOnAccent,
            onPressed: onPressed,
            text: text,
            iconLeft: Icon(icon),
        );
      },
    );
  }
}
