import 'package:flutter/material.dart';
import 'package:training_lights/models/training_model/training.dart';
import 'package:training_lights/my_theme.dart';
import 'package:training_lights/routes/countdown_route/countdown_controller.dart';
import 'package:training_lights/routes/countdown_route/widgets/countdown_timer_painter.dart';
import 'package:training_lights/utils/ratio_size.dart';

class CircularCountDownTimer extends StatefulWidget {
  final TrainingModel trainingModel;
  final CountdownController countDownController;
  final VoidCallback onComplete;

  const CircularCountDownTimer({
    required this.trainingModel,
    required this.countDownController,
    required this.onComplete,
  });

  @override
  CircularCountDownTimerState createState() => CircularCountDownTimerState();
}

class CircularCountDownTimerState extends State<CircularCountDownTimer>
    with TickerProviderStateMixin {
  late AnimationController animationController;
  late int round;

  @override
  void initState() {
    super.initState();
    _initRound();
    animationController = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: (widget.trainingModel.lengthWithPause * 1000).round()),
    );
    widget.countDownController.initCountdownController(animationController,
      runCountdownTimerAgain: () {
        _initRound();
        _animate();
      },
    );

    animationController.addStatusListener((status) {
      if (status == AnimationStatus.dismissed) {
        if (round == widget.trainingModel.repetition) {
          widget.onComplete();
        } else {
          round++;
          _animate();
        }
      }
    });

    _animate();
    widget.countDownController.pause();
  }

  @override
  void dispose() {
    animationController.stop();
    animationController.dispose();
    super.dispose();
  }

  int get time {
    if (animationController.isDismissed) {
      return 0;
    } else {
      final double duration = animationController.duration!.inSeconds * animationController.value;
      return duration.ceil();
    }
  }

  void _initRound() {
    round = 1;
  }

  void _animate() {
    animationController.reverse(from: 1);
  }

  @override
  Widget build(BuildContext context) {
    final theme = myTheme();
    final textStyle = theme.primaryTextTheme.headline1;
    final primaryColor = theme.primaryColor;
    final ratioSize = RatioSize.getRatioSizeOfSmallerSide(context, 0.5);

    return AnimatedBuilder(
      animation: animationController,
      builder: (context, child) {
        return Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Text(
                '$round/${widget.trainingModel.repetition}',
                style: textStyle,
            ),
            const SizedBox(height: 30),
            SizedBox(
                height: ratioSize,
                width: ratioSize,
                child: AspectRatio(
                  aspectRatio: 1.0,
                  child: Stack(
                    children: <Widget>[
                      Positioned.fill(
                        child: CustomPaint(
                          painter: CountdownTimerPainter(
                            animation: animationController,
                            fillColor: primaryColor,
                            ringColor: primaryColor.withOpacity(0.2),
                          ),
                        ),
                      ),
                      Align(
                        alignment: FractionalOffset.center,
                        child: Text(
                            '${time}s',
                            style: textStyle,
                        ),
                      )
                    ],
                  ),
                ),
            )
          ],
        );
      },
    );
  }
}
