import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:training_lights/common/dialogs/will_pop_dialog.dart';
import 'package:training_lights/common/wrappers/my_card.dart';
import 'package:training_lights/common/wrappers/my_scaffold.dart';
import 'package:training_lights/common/wrappers/my_snack_bar.dart';
import 'package:training_lights/global/notify_listeners/training_manager.dart';
import 'package:training_lights/models/training_model/training.dart';
import 'package:training_lights/models/training_model/training_element.dart';
import 'package:training_lights/my_theme.dart';
import 'package:training_lights/routes/create_training_route/numerical_values.dart';
import 'package:training_lights/routes/create_training_route/widgets/action_bottom_app_bar.dart';
import 'package:training_lights/routes/create_training_route/widgets/buttons/delete_training_button.dart';
import 'package:training_lights/routes/create_training_route/widgets/counter_row.dart';
import 'package:training_lights/routes/create_training_route/widgets/training_elements_list.dart';
import 'package:training_lights/routes/create_training_route/widgets/training_name_text_field.dart';
import 'package:training_lights/utils/languages/language.dart';
import 'package:training_lights/utils/logger.dart';

class CreateTrainingRouteArgs {
  final TrainingModel initialTraining;
  final bool editing;

  CreateTrainingRouteArgs({
    required this.initialTraining,
    required this.editing,
  });
}

class CreateTrainingRoute extends StatefulWidget {
  final CreateTrainingRouteArgs args;

  const CreateTrainingRoute({required this.args});

  @override
  CreateTrainingRouteState createState() => CreateTrainingRouteState();
}

class CreateTrainingRouteState extends State<CreateTrainingRoute> {
  final _log = getLogger('_CreatingTrainingRouteState');

  late bool areChangesSaved;
  late bool editing;
  late String name;
  late TrainingModel training;
  late int repetition;
  late int repetitionPause;
  late List<TrainingElement> trainingElements;
  late TextEditingController nameTextController;

  @override
  void initState() {
    super.initState();
    areChangesSaved = true;
    editing = widget.args.editing;
    training = widget.args.initialTraining;
    name = widget.args.initialTraining.name;
    repetition = widget.args.initialTraining.repetition;
    repetitionPause = widget.args.initialTraining.repetitionPause;
    trainingElements = deepCopyTrainingElement();
    nameTextController = TextEditingController(text: name);
  }

  List<TrainingElement> deepCopyTrainingElement() {
    return widget.args.initialTraining.trainingElements.map((trainingElement) {
      return trainingElement.copyWith();
    }).toList();
  }

  void wasSomethingEdited() {
    setState(() => areChangesSaved = false);
  }

  @override
  Widget build(BuildContext context) {
    final language = Language.of(context);
    const spacer = SizedBox(height: 10);
    final Widget willPopDialog = WillPopDialog(
      contentText: language.createTrainingRoutePopDialogLeaveRoute,
    );

    return MyScaffold(
        isBackButtonDisplayed: true,
        isBottomGradientDisplayed: false,
        sliverAppBar: true,
        willPopDialog: !areChangesSaved ? willPopDialog : null,// if something was edited then we have to show dialog before leaving
        bottomNavigationBar: ActionBottomAppBar(
            saveTraining: _saveTraining,
            areChangesSaved: areChangesSaved,
        ),
        title: Center(
          child: TrainingNameTextField(
            nameTextController: nameTextController,
            wasSomethingEdited: wasSomethingEdited,
          ),
        ),
        actionButton: editing ? DeleteTrainingButton(
            trainingModel: _createTrainingModel(),
        ) : null,
        body: Align(
          alignment: Alignment.topCenter, // because of when training is new
          child: SingleChildScrollView(
              child: Column(
                children: [
                  Align(
                    alignment: Alignment.topLeft,
                    child: Opacity(
                      opacity: 0.6,
                      child: Text(
                        language.createTrainingRouteDescription,
                        style: myTheme().textTheme.subtitle2,
                      ),
                    ),
                  ),
                  spacer,
                  MyCard(
                    child: CounterRow(
                      label: language.createTrainingRouteRepetitionNumber,
                      labelTextStyle: myTheme().textTheme.subtitle1,
                      value: repetition.toDouble(),
                      incrementValue: NumericalValues.repetitionIncrementValue,
                      maxValue: NumericalValues.repetitionMaxValue,
                      inSeconds: false,
                      editValue: (newValue) {
                        repetition = newValue.toInt();
                      },
                      wasSomethingEdited: wasSomethingEdited,
                    ),
                  ),
                  MyCard(
                    child: CounterRow(
                      label: language.createTrainingRouteRepetitionLengthOfPause,
                      labelTextStyle: myTheme().textTheme.subtitle1,
                      value: repetitionPause.toDouble(),
                      incrementValue: NumericalValues.repetitionRestIncrementValue,
                      maxValue: NumericalValues.repetitionRestMaxValue,
                      editValue: (newValue) {
                        repetitionPause = newValue.toInt();
                      },
                      wasSomethingEdited: wasSomethingEdited,
                    ),
                  ),
                  spacer,
                  TrainingElementsList(
                    trainingElements: trainingElements,
                    numberOfLights: widget.args.initialTraining.numberOfLights,
                    wasSomethingEdited: wasSomethingEdited,
                    spacer: spacer,
                  ),
                ],
              ),
          ),
        ),
    );
  }

  TrainingModel _createTrainingModel() {
    return TrainingModel(
        tid: training.tid,
        position: training.position,
        trainingElements: trainingElements,
        name: name,
        numberOfLights: widget.args.initialTraining.numberOfLights,
        repetition: repetition,
        repetitionPause: repetitionPause,
    );
  }

  TrainingModel _saveTraining(BuildContext context,
      {bool showSnackBar = true,}) {
    name = nameTextController.text.isEmpty ? name : nameTextController.text;
    final TrainingModel trainingModel = _createTrainingModel();

    if (editing) {
      context.read<TrainingManager>().update(trainingModel);
    } else {
      editing = true;
      context
          .read<TrainingManager>()
          .insert(trainingModel)
          .then((insertedTraining) => training = insertedTraining);
    }
    _log.d('Updating/Creating training with name: ${trainingModel.name}');

    setState(() {
      areChangesSaved = true;
    });

    if (showSnackBar) {
      ScaffoldMessenger.of(context).showSnackBar(
        MySnackBar(
          text: Language.of(context).createTrainingRouteTrainingSaved,
        ),
      );
    }

    return trainingModel;
  }

}
