import 'package:flutter/material.dart';
import 'package:training_lights/common/buttons/action_circle_button.dart';
import 'package:training_lights/common/my_icons/my_icons.dart';
import 'package:training_lights/models/training_model/blick.dart';
import 'package:training_lights/models/training_model/blick_pause.dart';
import 'package:training_lights/models/training_model/blick_prep.dart';
import 'package:training_lights/models/training_model/blick_random.dart';
import 'package:training_lights/models/training_model/training_element.dart';
import 'package:training_lights/my_theme.dart';
import 'package:training_lights/routes/create_training_route/widgets/cards/shape_card/shape_card.dart';
import 'package:training_lights/routes/create_training_route/widgets/cards/shape_card/shape_card_state.dart';
import 'package:training_lights/routes/create_training_route/widgets/cards/shape_card/shape_custom_painter.dart';
import 'package:training_lights/routes/create_training_route/widgets/cards/training_element_card_contents/blick_content.dart';
import 'package:training_lights/routes/create_training_route/widgets/cards/training_element_card_contents/pause_content.dart';
import 'package:training_lights/routes/create_training_route/widgets/cards/training_element_card_contents/preparation_content.dart';
import 'package:training_lights/utils/languages/language.dart';
import 'package:training_lights/utils/logger.dart';

class TrainingElementCard extends StatelessWidget {
  final _log = getLogger('Training element card');
  final int numberOfLights;
  final int index;
  final TrainingElement trainingElement;
  final List<TrainingElement> trainingElements;
  final Function(int) onDelete;
  final Function() wasSomethingEdited;
  final Function(int) showPlusElementButtons;
  final Function(BuildContext) scrollToAddedWidget;

  TrainingElementCard({
    required this.numberOfLights,
    required this.index,
    required this.trainingElement,
    required this.trainingElements,
    required this.onDelete,
    required this.wasSomethingEdited,
    required this.showPlusElementButtons,
    required this.scrollToAddedWidget,
  });

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        Builder(
          builder: (context) {
            scrollToAddedWidget(context);

            final Widget shapedCard = ShapeCard(
                shapeCardState: index == 0 ? ShapeCardState.withoutShape : ShapeCardState.attachShape,
                title: _trainingElementContent(context, isTitle: true),
                actionButton: ActionCircleButton(
                  onPressed: () => onDelete(index),
                  darkMode: true,
                  icon: MyIcons.crossSmall,
                ),
                body: _trainingElementContent(context),
            );

            // shape for plus button of last card
            if (index == trainingElements.length-1) {
              return Column(
                children: [
                  shapedCard,
                  CustomPaint(
                    size: Size(MediaQuery.of(context).size.width, 50),
                    painter: ShapeCustomPainter(
                      shapeCardState: ShapeCardState.onlyPlusButtonShape,
                    ),
                  )
                ],
              );
            } else {
              return shapedCard;
            }

          },
        ),
        if (index != 0)
          Positioned.fill(
            top: -14,
            child: _plusButton(Alignment.topCenter, index),
          ),
        if (index == trainingElements.length-1)
          Positioned.fill(
            bottom: 15,
            child: _plusButton(Alignment.bottomCenter, trainingElements.length),
          ),
      ],
    );
  }

  Widget _plusButton(Alignment alignment, int index) {
    return Align(
      alignment: alignment,
      child: Container(
        color: Colors.transparent,
        child: IconButton(
            color: myTheme().primaryColorLight,
            padding: EdgeInsets.zero,
            onPressed: () => showPlusElementButtons(index),
            icon: const Icon(MyIcons.plus),
        ),
      ),
    );
  }

  Widget _trainingElementContent(BuildContext context, {bool isTitle = false}) {
    final language = Language.of(context);

    switch (trainingElement.runtimeType) {
      case BlickModel:
        if (isTitle) {
          return _title(language.createTrainingRouteTrainingBlickCardBlick,
              _numberOfElement<BlickModel>(),);
        }
        return BlickContent(
          numberOfLights: numberOfLights,
          blick: trainingElement as BlickModel,
          wasSomethingEdited: wasSomethingEdited,
        );
      case BlickRandomModel:
        if (isTitle) {
          return _title(language.createTrainingRouteTrainingBlickCardRandom,
              _numberOfElement<BlickRandomModel>(),);
        }
        return BlickContent(
          numberOfLights: numberOfLights,
          blick: trainingElement as BlickRandomModel,
          wasSomethingEdited: wasSomethingEdited,
        );
      case BlickPrepModel:
        if (isTitle) {
          return _title(language.createTrainingRouteTrainingPrepCardPrep,
              _numberOfElement<BlickPrepModel>(),);
        }
        return PreparationContent(
          preparationModel: trainingElement as BlickPrepModel,
          wasSomethingEdited: wasSomethingEdited,
        );
      case BlickPauseModel:
        if (isTitle) {
          return _title(language.createTrainingRouteTrainingPauseCardPause,
              _numberOfElement<BlickPauseModel>(),);
        }
        return PauseContent(
          pauseModel: trainingElement as BlickPauseModel,
          wasSomethingEdited: wasSomethingEdited,
        );
      default:
        _log.e(
            'Unknown TrainingElement "${trainingElement.runtimeType}"',);
        return Container();
    }
  }

  Widget _title(String title, int numberOfElement) {
    return Row(
      children: [
        Text(
          title,
          style: myTheme().textTheme.subtitle1,
        ),
        const SizedBox(width: 5,),
        Opacity(
          opacity: 0.5,
          child: Text(
            '$numberOfElement',
            style: myTheme().textTheme.subtitle2,
          ),
        )
      ],
    );
  }

  int _numberOfElement<T>() {
    int number = 1;

    for (final trainingElementToCompare in trainingElements) {
      if (trainingElement == trainingElementToCompare) {
        break;
      }

      if (trainingElementToCompare is T) {
        if (!(trainingElementToCompare is BlickRandomModel && T == BlickModel)) {
          number++;
        }
      }
    }

    return number;
  }
}
