import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:training_lights/common/buttons/connection_state_button.dart';
import 'package:training_lights/common/buttons/play_training_button.dart';
import 'package:training_lights/common/buttons/primary_button.dart';
import 'package:training_lights/common/video/video_player_screen.dart';
import 'package:training_lights/common/wrappers/my_scaffold.dart';
import 'package:training_lights/global/enums/primary_button_design.dart';
import 'package:training_lights/models/training_model/recommended_training_model/recommended_training.dart';
import 'package:training_lights/my_theme.dart';
import 'package:training_lights/routes/about_us_route/about_us_route.dart';
import 'package:training_lights/routes/create_training_route/create_training_route.dart';
import 'package:training_lights/utils/languages/language.dart';

class RecommendedTrainingRoute extends StatelessWidget {
  final RecommendedTraining recommendedTraining;

  const RecommendedTrainingRoute({
    required this.recommendedTraining,
  });

  @override
  Widget build(BuildContext context) {
    final appBarWidgetsBackgroundColor = myTheme().primaryColor.withOpacity(0.6);

    return Column(
      children: [
        Expanded(
          child: MyScaffold(
            isBackButtonDisplayed: true,
            isBottomGradientDisplayed: false,
            sliverAppBar: true,
            backgroundSliverAppBar: FlexibleSpaceBar(
              background: VideoPlayerScreen(
                videoKey: recommendedTraining.key,
                inAppBar: true,
              ),
            ),
            title: ConnectionStateButton(
              backgroundColor: appBarWidgetsBackgroundColor,
            ),
            appBarWidgetsBackgroundColor: appBarWidgetsBackgroundColor,
            body: ListView(
              children: [
                _header(context),
                _content(context),
              ],
            ),
          ),
        ),
        Container(
          alignment: Alignment.center,
          padding: const EdgeInsets.only(bottom: 40, top: 10, left: 20, right: 20),
          child: PlayTrainingButton(training: recommendedTraining.trainingModel),
        ),
      ],
    );
  }

  Widget _header(BuildContext context) {
    final theme = myTheme();
    final language = Language.of(context);

    return Container(
      constraints: BoxConstraints(
        minHeight: MediaQuery.of(context).size.height * 0.07,
      ),
      margin: const EdgeInsets.symmetric(vertical: 20),
      child: Row(
        children: [
          Expanded(
            flex: 2,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text(
                  recommendedTraining.name,
                  style: theme.textTheme.headline2,
                ),
                Text(
                  '${recommendedTraining.trainingModel.numberOfLights} ${language.stripes}',
                  style: theme.textTheme.subtitle2,
                ),
              ],
            ),
          ),
          Expanded(
            child: PrimaryButton(
              onPressed: () {
                Navigator.of(context).pushNamed(
                    '/createTraining',
                    arguments: CreateTrainingRouteArgs(
                        initialTraining: recommendedTraining.trainingModel,
                        editing: false,
                    ),
                );
              },
              buttonDesign: PrimaryButtonDesign.accentColorOnDark,
              text: language.edit,
            ),
          )
        ],
      ),
    );
  }

  Widget _content(BuildContext context) {
    final language = Language.of(context);
    final TextStyle? textStyle = myTheme().textTheme.bodyText2;
    const spacer = SizedBox(height: 15,);

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Text(recommendedTraining.description, style: textStyle),
        spacer,
        Text(language.recommendedTrainingRouteSetup, style: textStyle),
        Padding(
            padding: const EdgeInsets.only(left: 10,top: 5),
            child: _textWithLink(context, language, textStyle),
        ),
        spacer,
        Text(language.recommendedTrainingRouteProcess, style: textStyle),
        Padding(
          padding: const EdgeInsets.only(left: 10,top: 5),
          child: Text(recommendedTraining.process, style: textStyle),
        ),
        spacer,
        Text(language.recommendedTrainingRouteEditExplanation, style: textStyle),
      ],
    );
  }

  Widget _textWithLink(BuildContext context, Language language, TextStyle? textStyle) {
    final List<String> setupParts = recommendedTraining.setup.split('@LINK@');

    if (setupParts.length > 1) {
      return Text.rich(
        TextSpan(
            children: [
              TextSpan(
                text: setupParts[0],
              ),
              TextSpan(
                  text: language.recommendedTrainingRouteHere,
                  style: TextStyle(
                      color: myTheme().colorScheme.secondary,
                  ),
                  recognizer: TapGestureRecognizer()
                    ..onTap = () {
                      Navigator.of(context).pushNamed(
                          '/aboutUs',
                          arguments: AboutUsRouteArgs(scrollDown: true),
                      );
                    },
              ),
              TextSpan(
                  text: setupParts[1],
              )
            ],
        ),
        style: textStyle,
      );
    } else {
      return Text(recommendedTraining.setup, style: textStyle);
    }
  }
}
