import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:training_lights/common/wrappers/my_color_picker.dart';
import 'package:training_lights/global/enums/tangle_color.dart';
import 'package:training_lights/global/notify_listeners/tangle_manager/tangle_manager.dart';
import 'package:training_lights/my_theme.dart';
import 'package:training_lights/routes/tangle_control_route/tangle_control_card_state.dart';
import 'package:training_lights/utils/languages/language.dart';

class TangleControlCardBody extends StatefulWidget {
  final TangleControlCardState cardState;
  final int lightNumber;
  final int numberOfLights;
  final TangleColor color;
  final void Function(int? newLightNumber) setLightNumber;
  final void Function(TangleColor newColor) setColor;

  const TangleControlCardBody({
    required this.cardState,
    required this.lightNumber,
    required this.numberOfLights,
    required this.color,
    required this.setLightNumber,
    required this.setColor,
  });

  @override
  State<StatefulWidget> createState() => _TangleControlCardBody();
}

class _TangleControlCardBody extends State<TangleControlCardBody>
    with TickerProviderStateMixin {
  late AnimationController animationController;
  Widget? child;

  @override
  void initState() {
    const duration = Duration(milliseconds: 100);
    animationController = AnimationController(
      vsync: this,
      duration: duration,
      reverseDuration: duration,
    );
    animationController.addListener(() => setState(() {}));
    super.initState();
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }

  void _animate(bool onlyControlButton) {
    if (!onlyControlButton) {
      animationController.forward();
    }
  }

  @override
  Widget build(BuildContext context) {
    final theme = myTheme();
    final bool onlyControlButton = widget.cardState == TangleControlCardState.onlyControlButton;

    _animate(onlyControlButton);

    if (widget.cardState == TangleControlCardState.colorSelection) {
      child = _colorSelectionBody();
    } else if (widget.cardState == TangleControlCardState.lightSelection) {
      child = _lightSelectionBody(theme);
    }

    return DecoratedBox(
      decoration: BoxDecoration(
        border: !onlyControlButton ? Border(
          top: BorderSide(
            color: theme.backgroundColor,
          ),
        ) : null,
      ),
      child: Column(
        children: [
          Expanded(
            flex: IntTween(begin: 0, end: 5).animate(animationController).value,
            child: SizedBox(
              height: onlyControlButton ? 0 : null,
              child: TweenAnimationBuilder(
                tween: Tween<double>(
                    begin: 0,
                    end: onlyControlButton ? 0 : 1,
                ),
                duration: Duration(
                    milliseconds: onlyControlButton ? 100 : 300,
                ),
                builder: (context, scale, child) {
                  return Transform.scale(scale: scale! as double, child: child,);
                },
                onEnd: () {
                  if (onlyControlButton) {
                    animationController.reverse();
                  }
                },
                child: child,
              ),
            ),
          ),
          Expanded(
            child: _controlButton(),
          )
        ],
      ),
    );
  }

  Widget _colorSelectionBody() {
    return SingleChildScrollView(
        child: MyColorPicker(
          color: widget.color,
          setColor: widget.setColor,
        ),
    );
  }

  Widget _lightSelectionBody(ThemeData theme) {
    return ListView.builder(
      shrinkWrap: true,
      itemCount: widget.numberOfLights,
      itemBuilder: (BuildContext context, int index) {
        return RadioListTile<int>(
          dense: true,
          title: Text(
              '${Language.of(context).stripe} ${index+1}',
              style: theme.textTheme.subtitle2,
          ),
          value: index+1,
          groupValue: widget.lightNumber,
          activeColor: theme.highlightColor,
          contentPadding: const EdgeInsets.only(left: 12, right: 4),
          controlAffinity: ListTileControlAffinity.trailing,
          onChanged: (int? newLightNumber) {
            widget.setLightNumber(newLightNumber);
          },
        );
      },
    );
  }

  Widget _controlButton() {
    return Consumer<TangleManager>(
        builder: (_, tangle, __) {
          return Listener(
            onPointerDown: (_) => tangle.lightOn(widget.lightNumber - 1, widget.color),
            onPointerUp: (_) => tangle.lightOff(widget.lightNumber - 1),
            child: ElevatedButton(
                style: ElevatedButton.styleFrom(
                  primary: widget.color.materialColor,
                  minimumSize: Size.infinite,
                  shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(15),
                  ),
                ),
                onPressed: () {},
                child: Container(height: 0),
            ),
          );
        },
    );
  }
}
